/********************************************************
 * Data Visualization - 2D Matrix Demo                  *
 * Tested in Processing 3                               *     
 *                                                      *
 * Author: Rodger (Jieliang) Luo                        *
 * Supervisor: George Legrady                           *
 *                                                      *
 * Purpose: Demonstrate how to make a simple 2D Matrix  *
 *          data visualization.                         *
 ********************************************************/

// Layout variables 
float constantHorizontalMargin = 160;
float constantVerticalMargin = 100;

// 2D matrix table related variables
Table table;
int numRows, numColumns;
float[][] dataMatrix;
float maxValue, minValue;
int dewey;
float DeweyWidth = 11;
PFont myFont;


// width and height for each cell
float cellWidth, cellHeight;

// text labels
int startYear = 2006;

boolean normalization = false;
boolean colorApplied = false; 
boolean reorder = false; 

void setup() {
  size(1280, 720); // setup the size of the window
  // In Processing 3, the size() function must be the first line of code inside setup()
  // and needs to use numbers instead of variables. 
   String[] fontList = PFont.list();
  printArray(fontList);
  myFont = createFont("Consola.ttf",5);
  smooth();
  // Make the screen resizable.                     
  surface.setResizable(true);
  
  //Put data into Processing table from csv
  table = loadTable("techdata.csv", "header");
  numRows = table.getRowCount(); 
  numColumns = table.getColumnCount();
  dataMatrix = new float[numColumns-1][numRows];
  println("Rows: " + numRows + " Columns: " + numColumns);
  

  for (int i=0; i<numColumns-1; i++) {
    for (int j=0; j<numRows; j++) {
      dataMatrix[i][j] = table.getFloat(j, i+1);
    }
  }

  maxValue = max2D(dataMatrix);
  minValue = min2D(dataMatrix);

  println("Max Value is " + maxValue);
  println("Min Value is " + minValue);
}

void draw() {

  //refresh the canvas every frame
  background(50,50,50);
 // font = loadFont("Consola.ttf");

  //modifying the cell size according to window size
  cellWidth = (width - constantHorizontalMargin*2)/numRows;
  cellHeight = (height - constantVerticalMargin*2)/(numColumns-1);

  //Draw 2D Matrix
  for (int i=0; i<numColumns-1; i++) {
    for (int j=0; j<numRows; j++) {
      //stroke(255);
      noStroke();
      //strokeWeight(1);
      float greyScale = map(dataMatrix[i][j], minValue, maxValue, 255, 0);
      fill(greyScale);
      //color c = colorBar.get((int)map(dataMatrix[i][j], minValue, maxValue, 2, colorBar.width-2), colorBar.height/2);
      ellipseMode(CORNER);
      ellipse(constantHorizontalMargin + cellWidth*j, constantVerticalMargin 
        + cellHeight*i, dataMatrix[i][j], dataMatrix[i][j]*4);
    }
  }
  
  //hover 2D Matrix
  for (int i=0; i <=numRows-1; i++){
  noFill();
  noStroke();
  rectMode(CORNER);
  rect(constantHorizontalMargin+cellWidth*i+2,
   height*.89-12, 8, 18);
  }

  
  //use println(mouseX, mouseY); to figure out the coordinates to choose where i can hover...
  
  //look into case language for hovering functions...processing.org

  //Draw vertical labels
  for (int i=0; i<numColumns-1; i++) {
    textFont(myFont);
    textAlign(RIGHT, CENTER);
    fill(57,255,20);
    textSize(15);
    text(startYear+i, constantHorizontalMargin-8, 
      constantVerticalMargin + cellHeight/2 + cellHeight*i-12);
  }
  
  //draw horizontal labels
  for (int j = 0; j<numRows; j++){
      float x = constantHorizontalMargin+5+cellWidth*j;
      float y = height*.88;
     textFont(myFont);
     textAlign(LEFT, CENTER);
     fill(57,255,20);
     textSize(12);
     pushMatrix();
     translate(x,
     y);
     rotate(3*PI/2);
     translate(-(x), -(y));
     text(Dewey[j], x, y);
   
     //constantHorizontalMargin + cellWidth/2 + cellWidth*j 
      //constantHorizontalMargin + cellWidth/2 + cellWidth*j
      //height*.90 
     popMatrix();
  }

  //Draw Title
  textFont(myFont);
  textAlign(CENTER, CENTER);
  textSize(30);
  text("technology's_relevance_through_the_years", width/2, constantVerticalMargin/2);
  
  //Draw Horizontal Title
  textFont(myFont);
  textAlign(CENTER, CENTER);
  textSize(15);
  text("dewey_classification_numbers", width/2, height*.95);
  
  //Draw vertical title
  textFont(myFont);
  textAlign(CENTER, CENTER);
  textSize(15);
  pushMatrix();
  translate(constantHorizontalMargin/2, 
      height/2);
  rotate(3*PI/2);
  translate(-(constantHorizontalMargin/2),-(height/2));
  text("year", constantHorizontalMargin/2, 
      height/2);
  popMatrix();
  
  if (normalization == true) normalization();
  else if(normalization == false) resetData(); 
  
  if (reorder == true) reorder();
  
  if(colorApplied == true) colorScheme();
  
  println(mouseX,mouseY);
  
  //650 - 630 for Y same for all coordinates
  // 1-8 for X value start with 162-170
  
  
}